@@ -3,5 +3,5 @@ from django.utils.translation import gettext_lazy as _
 
 
 class AdminDocsConfig(AppConfig):
-    name = 'django.contrib.admindocs'
+    name = "django.contrib.admindocs"
     verbose_name = _("Administrative Documentation")
