@@ -2,18 +2,22 @@
 Base file upload handler classes, and the built-in concrete subclasses
 """
 
+import os
 from io import BytesIO
 
 from django.conf import settings
-from django.core.files.uploadedfile import (
-    InMemoryUploadedFile, TemporaryUploadedFile,
-)
+from django.core.files.uploadedfile import InMemoryUploadedFile, TemporaryUploadedFile
 from django.utils.module_loading import import_string
 
 __all__ = [
-    'UploadFileException', 'StopUpload', 'SkipFile', 'FileUploadHandler',
-    'TemporaryFileUploadHandler', 'MemoryFileUploadHandler', 'load_handler',
-    'StopFutureHandlers'
+    "UploadFileException",
+    "StopUpload",
+    "SkipFile",
+    "FileUploadHandler",
+    "TemporaryFileUploadHandler",
+    "MemoryFileUploadHandler",
+    "load_handler",
+    "StopFutureHandlers",
 ]
 
 
@@ -21,6 +25,7 @@ class UploadFileException(Exception):
     """
     Any error having to do with uploading files.
     """
+
     pass
 
 
@@ -28,6 +33,7 @@ class StopUpload(UploadFileException):
     """
     This exception is raised when an upload must abort.
     """
+
     def __init__(self, connection_reset=False):
         """
         If ``connection_reset`` is ``True``, Django knows will halt the upload
@@ -38,15 +44,16 @@ class StopUpload(UploadFileException):
 
     def __str__(self):
         if self.connection_reset:
-            return 'StopUpload: Halt current upload.'
+            return "StopUpload: Halt current upload."
         else:
-            return 'StopUpload: Consume request data, then halt.'
+            return "StopUpload: Consume request data, then halt."
 
 
 class SkipFile(UploadFileException):
     """
     This exception is raised by an upload handler that wants to skip a given file.
     """
+
     pass
 
 
@@ -55,6 +62,7 @@ class StopFutureHandlers(UploadFileException):
     Upload handlers that have handled a file and do not want future handlers to
     run should raise this exception instead of returning None.
     """
+
     pass
 
 
@@ -62,7 +70,8 @@ class FileUploadHandler:
     """
     Base class for streaming upload handlers.
     """
-    chunk_size = 64 * 2 ** 10  # : The default chunk size is 64 KB.
+
+    chunk_size = 64 * 2**10  # : The default chunk size is 64 KB.
 
     def __init__(self, request=None):
         self.file_name = None
@@ -72,7 +81,9 @@ class FileUploadHandler:
         self.content_type_extra = None
         self.request = request
 
-    def handle_raw_input(self, input_data, META, content_length, boundary, encoding=None):
+    def handle_raw_input(
+        self, input_data, META, content_length, boundary, encoding=None
+    ):
         """
         Handle the raw input from the client.
 
@@ -90,7 +101,15 @@ class FileUploadHandler:
         """
         pass
 
-    def new_file(self, field_name, file_name, content_type, content_length, charset=None, content_type_extra=None):
+    def new_file(
+        self,
+        field_name,
+        file_name,
+        content_type,
+        content_length,
+        charset=None,
+        content_type_extra=None,
+    ):
         """
         Signal that a new file has been started.
 
@@ -109,7 +128,9 @@ class FileUploadHandler:
         Receive data from the streamed upload parser. ``start`` is the position
         in the file of the chunk.
         """
-        raise NotImplementedError('subclasses of FileUploadHandler must provide a receive_data_chunk() method')
+        raise NotImplementedError(
+            "subclasses of FileUploadHandler must provide a receive_data_chunk() method"
+        )
 
     def file_complete(self, file_size):
         """
@@ -118,7 +139,9 @@ class FileUploadHandler:
 
         Subclasses should return a valid ``UploadedFile`` object.
         """
-        raise NotImplementedError('subclasses of FileUploadHandler must provide a file_complete() method')
+        raise NotImplementedError(
+            "subclasses of FileUploadHandler must provide a file_complete() method"
+        )
 
     def upload_complete(self):
         """
@@ -127,17 +150,27 @@ class FileUploadHandler:
         """
         pass
 
+    def upload_interrupted(self):
+        """
+        Signal that the upload was interrupted. Subclasses should perform
+        cleanup that is necessary for this handler.
+        """
+        pass
+
 
 class TemporaryFileUploadHandler(FileUploadHandler):
     """
     Upload handler that streams data into a temporary file.
     """
+
     def new_file(self, *args, **kwargs):
         """
         Create the file object to append to as data is coming in.
         """
         super().new_file(*args, **kwargs)
-        self.file = TemporaryUploadedFile(self.file_name, self.content_type, 0, self.charset, self.content_type_extra)
+        self.file = TemporaryUploadedFile(
+            self.file_name, self.content_type, 0, self.charset, self.content_type_extra
+        )
 
     def receive_data_chunk(self, raw_data, start):
         self.file.write(raw_data)
@@ -147,13 +180,24 @@ class TemporaryFileUploadHandler(FileUploadHandler):
         self.file.size = file_size
         return self.file
 
+    def upload_interrupted(self):
+        if hasattr(self, "file"):
+            temp_location = self.file.temporary_file_path()
+            try:
+                self.file.close()
+                os.remove(temp_location)
+            except FileNotFoundError:
+                pass
+
 
 class MemoryFileUploadHandler(FileUploadHandler):
     """
     File upload handler to stream uploads into memory (used for small files).
     """
 
-    def handle_raw_input(self, input_data, META, content_length, boundary, encoding=None):
+    def handle_raw_input(
+        self, input_data, META, content_length, boundary, encoding=None
+    ):
         """
         Use the content_length to signal whether or not this handler should be
         used.
@@ -188,7 +232,7 @@ class MemoryFileUploadHandler(FileUploadHandler):
             content_type=self.content_type,
             size=file_size,
             charset=self.charset,
-            content_type_extra=self.content_type_extra
+            content_type_extra=self.content_type_extra,
         )
 
 
@@ -199,7 +243,10 @@ def load_handler(path, *args, **kwargs):
     E.g.::
         >>> from django.http import HttpRequest
         >>> request = HttpRequest()
-        >>> load_handler('django.core.files.uploadhandler.TemporaryFileUploadHandler', request)
+        >>> load_handler(
+        ...     'django.core.files.uploadhandler.TemporaryFileUploadHandler',
+        ...     request,
+        ... )
         <TemporaryFileUploadHandler object at 0x...>
     """
     return import_string(path)(*args, **kwargs)
