@@ -1,36 +1,40 @@
-import asyncio
 import inspect
 import warnings
 
-from asgiref.sync import sync_to_async
+from asgiref.sync import iscoroutinefunction, markcoroutinefunction, sync_to_async
 
 
-class RemovedInDjango40Warning(DeprecationWarning):
+class RemovedInNextVersionWarning(DeprecationWarning):
     pass
 
 
-class RemovedInDjango41Warning(PendingDeprecationWarning):
+class RemovedInDjango60Warning(PendingDeprecationWarning):
     pass
 
 
-RemovedInNextVersionWarning = RemovedInDjango40Warning
+RemovedAfterNextVersionWarning = RemovedInDjango60Warning
 
 
 class warn_about_renamed_method:
-    def __init__(self, class_name, old_method_name, new_method_name, deprecation_warning):
+    def __init__(
+        self, class_name, old_method_name, new_method_name, deprecation_warning
+    ):
         self.class_name = class_name
         self.old_method_name = old_method_name
         self.new_method_name = new_method_name
         self.deprecation_warning = deprecation_warning
 
     def __call__(self, f):
-        def wrapped(*args, **kwargs):
+        def wrapper(*args, **kwargs):
             warnings.warn(
-                "`%s.%s` is deprecated, use `%s` instead." %
-                (self.class_name, self.old_method_name, self.new_method_name),
-                self.deprecation_warning, 2)
+                "`%s.%s` is deprecated, use `%s` instead."
+                % (self.class_name, self.old_method_name, self.new_method_name),
+                self.deprecation_warning,
+                2,
+            )
             return f(*args, **kwargs)
-        return wrapped
+
+        return wrapper
 
 
 class RenameMethodsBase(type):
@@ -63,9 +67,11 @@ class RenameMethodsBase(type):
                 # Define the new method if missing and complain about it
                 if not new_method and old_method:
                     warnings.warn(
-                        "`%s.%s` method should be renamed `%s`." %
-                        (class_name, old_method_name, new_method_name),
-                        deprecation_warning, 2)
+                        "`%s.%s` method should be renamed `%s`."
+                        % (class_name, old_method_name, new_method_name),
+                        deprecation_warning,
+                        2,
+                    )
                     setattr(base, new_method_name, old_method)
                     setattr(base, old_method_name, wrapper(old_method))
 
@@ -80,7 +86,8 @@ class DeprecationInstanceCheck(type):
     def __instancecheck__(self, instance):
         warnings.warn(
             "`%s` is deprecated, use `%s` instead." % (self.__name__, self.alternative),
-            self.deprecation_warning, 2
+            self.deprecation_warning,
+            2,
         )
         return super().__instancecheck__(instance)
 
@@ -89,33 +96,38 @@ class MiddlewareMixin:
     sync_capable = True
     async_capable = True
 
-    # RemovedInDjango40Warning: when the deprecation ends, replace with:
-    #   def __init__(self, get_response):
-    def __init__(self, get_response=None):
-        self._get_response_none_deprecation(get_response)
+    def __init__(self, get_response):
+        if get_response is None:
+            raise ValueError("get_response must be provided.")
         self.get_response = get_response
-        self._async_check()
+        # If get_response is a coroutine function, turns us into async mode so
+        # a thread is not consumed during a whole request.
+        self.async_mode = iscoroutinefunction(self.get_response)
+        if self.async_mode:
+            # Mark the class as async-capable, but do the actual switch inside
+            # __call__ to avoid swapping out dunder methods.
+            markcoroutinefunction(self)
         super().__init__()
 
-    def _async_check(self):
-        """
-        If get_response is a coroutine function, turns us into async mode so
-        a thread is not consumed during a whole request.
-        """
-        if asyncio.iscoroutinefunction(self.get_response):
-            # Mark the class as async-capable, but do the actual switch
-            # inside __call__ to avoid swapping out dunder methods
-            self._is_coroutine = asyncio.coroutines._is_coroutine
+    def __repr__(self):
+        return "<%s get_response=%s>" % (
+            self.__class__.__qualname__,
+            getattr(
+                self.get_response,
+                "__qualname__",
+                self.get_response.__class__.__name__,
+            ),
+        )
 
     def __call__(self, request):
         # Exit out to async mode, if needed
-        if asyncio.iscoroutinefunction(self.get_response):
+        if self.async_mode:
             return self.__acall__(request)
         response = None
-        if hasattr(self, 'process_request'):
+        if hasattr(self, "process_request"):
             response = self.process_request(request)
         response = response or self.get_response(request)
-        if hasattr(self, 'process_response'):
+        if hasattr(self, "process_response"):
             response = self.process_response(request, response)
         return response
 
@@ -125,23 +137,15 @@ class MiddlewareMixin:
         is running.
         """
         response = None
-        if hasattr(self, 'process_request'):
+        if hasattr(self, "process_request"):
             response = await sync_to_async(
                 self.process_request,
                 thread_sensitive=True,
             )(request)
         response = response or await self.get_response(request)
-        if hasattr(self, 'process_response'):
+        if hasattr(self, "process_response"):
             response = await sync_to_async(
                 self.process_response,
                 thread_sensitive=True,
             )(request, response)
         return response
-
-    def _get_response_none_deprecation(self, get_response):
-        if get_response is None:
-            warnings.warn(
-                'Passing None for the middleware get_response argument is '
-                'deprecated.',
-                RemovedInDjango40Warning, stacklevel=3,
-            )
